//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     rules.cxx
// Purpose:    List of rules
// Last Change:03.10.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2001 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "out.h"
#include "rule.h"

#include "rules.h"

//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// Constructor:
//-----------------------------------------------------------------------------

rules_c::rules_c(rule_t head)
{
	// Check Parameter:
	CHECK(head != 0, "rules_c::Constructor: Head is NIL");
	CHECK(head->valid(), "rules_c::Constructor: Head is invalid");

	// Initialize Magic Number:
#ifdef VER_DEBUG
	Magic = RULES_MAGIC;
#endif

	// Initialize object:
	Head = head;
	Tail = 0;
}

//-----------------------------------------------------------------------------
// Destructor:
//-----------------------------------------------------------------------------

rules_c::~rules_c(void)
{
#ifdef	VER_DEBUG
	Magic = 0;
#endif
}

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
bool_t	rules_c::valid(void) const
{
	// Check object pointer:
	CHECK(this != 0, "rules_c::valid: NIL object");

	// Magic number must be correct:
	if(Magic != RULES_MAGIC)
		return(BOOL_FALSE);

	// Head must be not null:
	if(Head == 0)
		return(BOOL_FALSE);

	// Head must be valid:
	if(Head->valid() != BOOL_TRUE)
		return(BOOL_FALSE);

	// Ok:
	return(BOOL_TRUE);
}
#endif

//-----------------------------------------------------------------------------
// Print this term list:
//-----------------------------------------------------------------------------

void rules_c::print(void) const
{
	rules_c *p;

	// Check Consistency:
	CHECK(valid(), "rules_c::print: Bad object");

	// Loop over rules, print them as unordered list:
	out_c::list_begin();
	out_c::list_item();
	Head->print();
	out_c::print_nl();
	for(p = Tail; p != 0; p = p->get_tail()) {
		out_c::list_item();
		p->get_head()->print();
		out_c::print_nl();
	}
	out_c::list_end();
}

